<?php
/**
 * REST API Payment controller
 *
 * Handles requests to the /payment endpoint.
 *
 * @author   QWQ
 * @category API
 * @package  WooCommerce/API
 * @since    2.6.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * REST API Payment controller class.
 *
 * @package WooCommerce/API
 * @extends WC_REST_Posts_Controller
 */
class W2W_REST_Payment_Controller extends W2W_REST_Posts_Controller {

	/**
	 * Endpoint namespace.
	 *
	 * @var string
	 */
	protected $namespace = 'w2w/v1';

	/**
	 * Route base.
	 *
	 * @var string
	 */
	protected $rest_base = 'payment';

	
	/**
	 * Initialize payment actions.
	 */
	public function __construct() {
		WC()->payment_gateways();
	}

	/**
	 * Register the routes for payment.
	 */
	public function register_routes() {
		
		register_rest_route( $this->namespace, '/' . $this->rest_base . '/', array(
			array(
				'methods'             => WP_REST_Server::READABLE,
				'callback'            => array( $this, 'get_payment_params' ),
				'permission_callback' => array( $this, 'get_items_permissions_check' ),
				'args'                => $this->get_collection_params(),
			)
		) );

		register_rest_route( $this->namespace, '/' . $this->rest_base . '/notify', array(
			array(
				'methods'             => WP_REST_Server::CREATABLE,
				'callback'            => array( $this, 'notify' ),
				'permission_callback' => array( $this, 'get_items_permissions_check' ),
				'args'                => $this->get_collection_params(),
			)
		) );
	}
	
	/**
	 * Check whether a given request has permission to read order notes.
	 *
	 * @param  WP_REST_Request $request Full details about the request.
	 * @return WP_Error|boolean
	 */
	public function get_items_permissions_check( $request ) {
		return true;
	}
	
	// 获取支付参数
	public function get_payment_params( $request ) {
		
		$session = W2W()->util->get_session( $request['w2w_session'] );
		if( ! $session ) return array( 'success' => false, 'msg' => 'Session过期' );
		
		$wc_order = wc_get_order( $request['id'] );
		if( ! $wc_order ) return false;
		
		$params = apply_filters( 'w2w_get_payment_params', false, $wc_order );
		
		if( is_array( $params ) && isset( $params['success'], $params['msg'] ) && ! $params['success'] ) {
			return array( 'success' => false, 'msg' => $params['msg'] );
		}
		elseif( ! $params ) {
			return array( 'success' => false, 'msg' => '支付网关没有正确返回支付参数' );
		}
		else {
			return $params;
		}
	}
	
	// 支付通知
	public function notify( $request ) {
		
		W2W()->log( 'info', '支付通知开始' );
		
		do_action( 'w2w_payment_notify', $request );
	}
}
