<?php

if ( ! defined( 'ABSPATH' ) ) exit;
		
// 适配WooCommerce Dynamic Pricing
add_action( 'wp_loaded', 'w2w_remove_wdp_on_cart_loaded_from_session', 1 );
function w2w_remove_wdp_on_cart_loaded_from_session() {
	if( class_exists('WC_Dynamic_Pricing') && W2W()->is_in_miniprogram() ) {
		remove_action( 'woocommerce_cart_loaded_from_session', array( WC_Dynamic_Pricing::instance(), 'on_cart_loaded_from_session' ), 98, 1 );
	}
}


// 适配Wechat Social 设置项
add_action( 'w2w_settings_advanced_tab_content_end', 'w2w_xhws_compatibility_setting' );
function w2w_xhws_compatibility_setting() {
	$settings = get_option( 'w2w-settings' );
	$xhws_plugin = sprintf( '<a href="%s" class="%s" aria-label="%s" data-title="%s">%s</a>',
		esc_url( network_admin_url( 'plugin-install.php?tab=plugin-information&plugin=wechat-social-login' .
			'&TB_iframe=true&width=600&height=550' ) ),
		"thickbox open-plugin-details-modal",
		esc_attr( '更多关于 Wechat Social 的信息' ),
		esc_attr( 'Wechat Social' ),
		'Wechat Social'
	);
	$installed = class_exists('XH_Social_Channel_Wechat');
	$adapted = false;
	if( $installed ) {
		global $wpdb;
		$columns = $wpdb->get_col("DESC `{$wpdb->prefix}xh_social_channel_wechat`", 0);
		$adapted = in_array( 'wa_openid', $columns );
	}
?>
<tr><th>适配迅虎Wechat Social</th><td>
<label for="w2w-settings[xhws_compatibility]" title="<?php if( ! $installed ) echo '请先安装Wechat Social' ?>">
<input type="checkbox" id="w2w-settings[xhws_compatibility]"
	   name="w2w-settings[xhws_compatibility]"
	   <?php checked( isset( $settings['xhws_compatibility'] ) ) ?>
	   <?php disabled( ! $installed , true ) ?>
>开启
<button id="xhws-compatibility" class="button button-default" <?php disabled( ! $installed || $adapted, true ) ?>><?php echo $adapted ? '数据表已适配' : '适配数据表' ?></button>
<p class="description">开启此功能可实现PC端、移动端、微信内置浏览器和小程序端中的微信用户同步，要正常使用此功能，请确保按以下步骤操作：</p>
</label>
<ul>
<li>1. 注册并认证<a href="https://open.weixin.qq.com/" target="_blank">微信开放平台</a></li>
<li>2. 关联公众号和小程序到微信开放平台</li>
<li>3. 安装 <?php echo $xhws_plugin ?> 插件</li>
<li>4. 转到 <a href="admin.php?page=social_page_default&section=menu_default_channel&sub=social_wechat" target="_blank">Wechat Social - 设置 - 社会化登录设置 - 微信</a>，切换到“移动端+PC端”选项卡，填写微信开放平台下的两项设置 <code>AppID</code> 和 <code>APP Secret</code></li>
<li>5. 如果你是 Wechat Social 付费版用户，请转到<a href="admin.php?page=social_page_default&section=menu_default_ext" target="_blank">Wechat Social - 设置 - 高级扩展</a>，登录模式选择“模式一”</li>
<li>6. 点击上方“适配数据表”，提示“数据表已适配”后，再勾选“开启”并保存设置</li>
</ul>
</td></tr>
<script>
jQuery(function($){
	$('#xhws-compatibility').click(function(){
		if( ! confirm('确定适配？') ) return false;
		var $btn = $(this);
		$btn.attr('disabled', 'disabled').text('正在适配，请稍候...');
		$.ajax({
			url: ajaxurl,
			type: 'POST',
			dataType: 'json',
			data: {
				action: 'w2w_alter_xh_table'
			},
			success: function(data) {
				if( data.success ) {
					$btn.attr('disabled', 'disabled').text('已适配，等待刷新...');
					location.reload();
				} else {
					alert( data.msg );
				}
			}
		});
	});
});
</script>
<style>
#xhws-compatibility {
    margin-left: 10px;
	vertical-align: middle;
}
</style>
<?php
}

// 适配Wechat Social Ajax修改表
add_action( 'wp_ajax_w2w_alter_xh_table', 'w2w_xhws_ajax_alter_xh_table' );
function w2w_xhws_ajax_alter_xh_table() {
	if( ! current_user_can('manage_options') ) {
		exit;
	}
	
	if( ! class_exists('XH_Social_Channel_Wechat') ) {
		wp_send_json( array( 'success' => false, 'msg' => '请先安装Wechat Social插件' ) );
	}
	
	global $wpdb;
	$columns = $wpdb->get_col("DESC `{$wpdb->prefix}xh_social_channel_wechat`", 0);
	if( in_array( 'wa_openid', $columns ) ) {
		wp_send_json( array( 'success' => false, 'msg' => '已经适配过了' ) );
	}
	
	$result = $wpdb->query("ALTER TABLE `{$wpdb->prefix}xh_social_channel_wechat` 
					ADD `wa_openid` VARCHAR(64) NULL DEFAULT NULL AFTER `mp_openid`,
					ADD UNIQUE `wa_openid_unique` (`wa_openid`);");
	$response = array( 'success' => $result === true );
	wp_send_json( $response );
	wp_die();
}

// 适配Wechat Social 用户登录
add_filter( 'w2w_before_login', 'w2w_xhws_create_xh_user_and_login', 10, 2 );
function w2w_xhws_create_xh_user_and_login( $response, $data ) {
	$settings = get_option( 'w2w-settings' );
	if( class_exists('XH_Social_Channel_Wechat') && isset( $data['unionId'] ) && isset( $settings['xhws_compatibility'] ) ) {
		$xh_wechat = XH_Social_Channel_Wechat::instance();
		$user = $xh_wechat->get_wp_user( 'unionid', $data['unionId'] );
		
		if( is_null( $user ) ) {
			$data['openid'] = $data['openId'];
			$data['nickname'] = $data['nickName'];
			$data['sex'] = $data['gender'];
			$data['headimgurl'] = $data['avatarUrl'];
			$data['unionid'] = $data['unionId'];
			$uid = XH_Social_Helper_String::guid();
			$ext_user_id = $xh_wechat->create_ext_user_info( 'wa', $data, 0, $uid );
			$xh_wechat->process_login( $ext_user_id, false );
			$user = $xh_wechat->get_wp_user( 'unionid', $data['unionId'] );
		}
		
		// 生成SESSION
		$w2w_session = W2W()->util->generate_session( $user->ID );
		W2W()->util->login( $w2w_session );
		
		do_action( 'w2w_login_with_user_info', $user->ID );
		
		$response = array(
			'user_id' => $user->ID,
			'w2w_session' => $w2w_session
		);
		$response = rest_ensure_response( $response );
	}
	return $response;
}

// 适配Wechat Social 获取OpenID
add_filter( 'w2w_get_user_openid', 'w2w_xhws_get_user_openid', 10, 2 );
function w2w_xhws_get_user_openid( $openid, $user ) {
	$settings = get_option( 'w2w-settings' );
	if( class_exists('XH_Social_Channel_Wechat') && isset( $settings['xhws_compatibility'] ) ) {
		global $wpdb;
		$result = $wpdb->get_row( $wpdb->prepare("SELECT * FROM `{$wpdb->prefix}xh_social_channel_wechat` w
													WHERE w.user_id = %d", $user->ID) );
		
		if( ! is_null( $result ) ) {
			$openid = $result->wa_openid;
		}
	}
	return $openid;
}