<?php

if ( ! defined( 'ABSPATH' ) ) exit;

include_once( WP_W2W_PATH . 'vendor/html2wxml/class.ToWXML.php' );
	
class W2W_Util {
	
	/**
	 * 检验数据的真实性，并且获取解密后的明文.
	 * @param $sessionKey string 用户在小程序登录后获取的会话密钥
	 * @param $appid string 小程序的appid
	 * @param $encryptedData string 加密的用户数据
	 * @param $iv string 与用户数据一同返回的初始向量
	 * @param $data string 解密后的原文
	 *
	 * @return int 成功0，失败返回对应的错误码
	 */
	public static function decrypt_data( $appid, $sessionKey, $encryptedData, $iv, &$data ) {
		
		$errors = array(
			'OK'				=> 0,
			'IllegalAesKey'		=> -41001,
			'IllegalIv'			=> -41002,
			'IllegalBuffer'		=> -41003,
			'DecodeBase64Error' => -41004
		);
		
		if (strlen($sessionKey) != 24)
		{
			return $errors['IllegalAesKey'];
		}
		$aesKey=base64_decode($sessionKey);

		
		if (strlen($iv) != 24)
		{
			return $errors['IllegalIv'];
		}
		$aesIV=base64_decode($iv);

		$aesCipher=base64_decode($encryptedData);

		$result=openssl_decrypt( $aesCipher, 'AES-128-CBC', $aesKey, 1, $aesIV);

		$dataObj=json_decode( $result );
		if( $dataObj  == NULL )
		{
			return $errors['IllegalBuffer'];
		}
		if( $dataObj->watermark->appid != $appid )
		{
			return $errors['IllegalBuffer'];
		}
		$data = $result;
		return $errors['OK'];
	}
	
	// 32位随机数
	public static function get_random() {
		return md5( uniqid( md5( microtime( true ) ), true ) );
	}
	
	// 生成Session
	public static function generate_session( $customer_id ) {
		
		$w2w_session = md5( uniqid( md5( microtime( true ) ), true ) );
		update_user_meta( $customer_id, 'w2w_session', $w2w_session );
		update_user_meta( $customer_id, 'w2w_session_expire', time() + 7200 );
		
		return $w2w_session;
	}
	
	// 根据 Session 获取 OpenID
	public static function get_session( $session_id ) {
		
		if( empty( $session_id ) ) {
			return false;
		}
		
		$user_query = new WP_User_Query( array( 'meta_key' => 'w2w_session', 'meta_value' => $session_id ) );
		$users = $user_query->get_results();
		
		if( ! empty( $users ) ) {
			$customer = $users[0];
			return array(
				'user_id' => $customer->ID,
				'openid' => self::get_user_openid( $customer ),
				'expire' => get_user_meta( $customer->ID, 'w2w_session_expire', true )
			);
		}
		
		return false;
	}

	// 根据 Session 登录对应用户
	public static function login( $session_id ) {
		
		$session = self::get_session( $session_id );
		if( $session ) {
			
			wp_set_current_user( $session['user_id'] );
			wc_set_customer_auth_cookie( $session['user_id'] );
			return true;
		}
		
		return false;
	}
	
	// 根据 UnionID 获取对应小程序用户
	public static function get_user_by_unionid( $unionid ) {
		
		if( empty( $unionid ) ) {
			return false;
		}
		
		$user_query = new WP_User_Query( array( 'meta_key' => 'w2w_unionid', 'meta_value' => $unionid ) );
		$users = $user_query->get_results();
		
		if( ! empty( $users ) ) {
			$customer = $users[0];
			return $customer;
		}
		
		return false;
	}
	
	// 获取用户 OpenID
	public static function get_user_openid( $user ) {
		return apply_filters( 'w2w_get_user_openid', $user->user_login, $user );
	}
	
	// 组合参数
	public static function param_atts( $pairs, $atts ) {
		
		$atts = (array)$atts;
		$out = array();
		foreach( $pairs as $name => $default ) {
			if( array_key_exists( $name, $atts ) ) {
				$out[$name] = $atts[$name];
			}    
			else {
				$out[$name] = $default;
			}
		}
		
		return $out;
	}
	
	// 检查付款通知签名
	public static function check_notify_sign( $data, $key ) {
		
		ksort( $data );
		$buff = '';
		
		foreach( $data as $k => $v ) {
			if( $k == 'sign' ||  is_array( $v ) ) continue;
			$buff .= $k . '=' . $v . '&';
		}
		
		$string_sign = $buff . 'key=' . $key;
		$sign = strtoupper( md5( $string_sign ) );
		
		if( $sign == $data['sign'] ) {
			
			return true;
		}
	}
	
	// 根据省代码获取名字
	public static function get_state_name( $country, $state ) {
		
		$states = WC()->countries->get_states();
		return isset( $states[$country][$state] ) ? $states[$country][$state] : '';
	}
	
	/**
	 * 删除指定标签
	 *
	 * @param array $tags     删除的标签  数组形式
	 * @param string $str     html字符串
	 * @param bool $content   true保留标签的内容text
	 * @return mixed
	 */
	public static function strip_html_tags( $tags, $str, $content = true ) {
		
		$html = array();
		foreach ($tags as $tag) {
			if( $content ) {
				$html[] = '/(<(?:\/' . $tag . '|' . $tag . ')[^>]*>)/is';
			}
			else{
				$html[] = '/(<' . $tag . '.*?>[\s|\S]*?<\/' . $tag . '>)/is';
			}
			
		}
		$data = preg_replace( $html, '', $str );
		return $data;
	}
	
	// 获取时间线格式的时间
	public static function get_timeline_date_format( $datetime ) {
		
		$date_diff = strtotime( date_i18n('Y-m-d') ) - strtotime( date( 'Y-m-d', strtotime( $datetime ) ) );
		$day = (int)( $date_diff / ( 3600 * 24 ) );
		//相差7天不作修改
		if( $day > 7 ) {
			return date( 'Y-m-d', strtotime( $datetime ) );
		}
		//...天前
		else if( $day >= 3 && $day <= 7 ) {
			return $day . ' 天前';
		}
		//前天...
		else if( $day == 2 ) {
			return ' 前天 '. date( 'H:i', strtotime( $datetime ) );
		}
		//昨天...
		else if( $day == 1 ) {
			return ' 昨天 ' . date( 'H:i', strtotime( $datetime ) );
		}
		//今天
		else if( $day < 1 ) {
			$time_diff = time() - strtotime( $datetime );
			$hour = (int)( ( $time_diff % ( 3600 * 24 ) ) / 3600 );
			$min = (int)( $time_diff % 3600 / 60 );
			$sec = (int)( $time_diff % 60 );
			//...小时前
			if( $hour >= 1 ) {
				return $hour . ' 小时前';
			}
			else {
				//半小时前
				if( $min >= 30 ) {
					return '半小时前';
				}
				//...分钟前
				else if( $min >= 1 && $min < 30 ) {
					return $min . ' 分钟前';
				}
				//...秒前
				else {
					return $sec . ' 秒前';
				}
			}
		}
	}
	
	// 产品下载
	/**
	 * Get the downloads for a product or product variation.
	 *
	 * @param WC_Product|WC_Product_Variation $product
	 * @return array
	 */
	public static function get_product_downloads( $product ) {
		$downloads = array();

		if ( $product->is_downloadable() ) {
			foreach ( $product->get_files() as $file_id => $file ) {
				$downloads[] = array(
					'id'   => $file_id, // MD5 hash.
					'name' => $file['name'],
					'file' => $file['file'],
				);
			}
		}

		return $downloads;
	}

	// 产品顺序
	/**
	 * Get product menu order.
	 *
	 * @param WC_Product $product
	 * @return int
	 */
	public static function get_product_menu_order( $product ) {
		$post_object = get_post( $product->get_id() );
		$menu_order = $post_object->menu_order;

		if ( $product->is_type( 'variation' ) ) {
			$variation  = get_post( $product->get_id() );
			$menu_order = $variation->menu_order;
		}

		return $menu_order;
	}
	
	// 获取产品数据
	/**
	 * Get product data.
	 *
	 * @param WC_Product $product
	 * @return array
	 */
	public static function get_product_data( $product, $type = 'single' ) {
		$settings = get_option( 'w2w-settings' );
		$related_products_quantity = ! empty( $settings['related_products_quantity'] ) ? absint( $settings['related_products_quantity'] ) : 4;
		
		$post_object = get_post( $product->get_id() );
		switch( $type ) {
			case 'single':
				$image_size = 'full';
				break;
			case 'list':
			case 'cart':
				$image_size = 'single';
				break;
			default:
				$image_size = 'full';
		}
		
		$data = array(
			'id'                    => $product->get_id(),
			'name'                  => $product->get_title(),
			'slug'                  => $post_object->post_name,
			//'permalink'             => $product->get_permalink(),
			'date_created'          => wc_rest_prepare_date_response( $post_object->post_date_gmt ),
			'date_modified'         => wc_rest_prepare_date_response( $post_object->post_modified_gmt ),
			'type'                  => $product->get_type(),
			'status'                => $post_object->post_status,
			'featured'              => $product->is_featured(),
			'catalog_visibility'    => $product->get_catalog_visibility(),
			//'description'           => wpautop( do_shortcode( $post_object->post_content ) ),
			//'short_description'     => apply_filters( 'woocommerce_short_description', $post_object->post_excerpt ),
			'sku'                   => $product->get_sku(),
			'price'                 => $product->get_price(),
			'regular_price'         => $product->get_regular_price(),
			'sale_price'            => $product->get_sale_price() ? $product->get_sale_price() : '',
			'date_on_sale_from'     => $product->get_date_on_sale_from() ? $product->get_date_on_sale_from()->date_i18n('Y-m-d H:i:s') : '',
			'date_on_sale_to'       => $product->get_date_on_sale_to() ? $product->get_date_on_sale_to()->date_i18n('Y-m-d H:i:s') : '',
			//'price_html'            => $product->get_price_html(),
			'on_sale'               => $product->is_on_sale(),
			'purchasable'           => $product->is_purchasable(),
			'total_sales'           => (int) get_post_meta( $product->get_id(), 'total_sales', true ),
			'virtual'               => $product->is_virtual(),
			'downloadable'          => $product->is_downloadable(),
			'download_limit'        => $product->get_download_limit(),
			'download_expiry'       => $product->get_download_expiry(),
			'download_type'         => 'standard',
			'external_url'          => $product->is_type( 'external' ) ? $product->get_product_url() : '',
			'button_text'           => $product->is_type( 'external' ) ? $product->get_button_text() : '',
			'tax_status'            => $product->get_tax_status(),
			'tax_class'             => $product->get_tax_class(),
			'manage_stock'          => $product->managing_stock(),
			'stock_quantity'        => $product->get_stock_quantity(),
			'in_stock'              => $product->is_in_stock(),
			'backorders'            => $product->get_backorders(),
			'backorders_allowed'    => $product->backorders_allowed(),
			'backordered'           => $product->is_on_backorder(),
			'sold_individually'     => $product->is_sold_individually(),
			'weight'                => $product->get_weight(),
			'dimensions'            => array(
				'length' => $product->get_length(),
				'width'  => $product->get_width(),
				'height' => $product->get_height(),
			),
			'shipping_required'     => $product->needs_shipping(),
			'shipping_taxable'      => $product->is_shipping_taxable(),
			'shipping_class'        => $product->get_shipping_class(),
			'shipping_class_id'     => (int) $product->get_shipping_class_id(),
			'reviews_allowed'       => ( 'open' === $post_object->comment_status ),
			'average_rating'        => wc_format_decimal( $product->get_average_rating(), 2 ),
			'rating_count'          => (int) $product->get_rating_count(),
			'related_ids'           => array_map( 'absint', array_values( wc_get_related_products( $product->get_id(), $related_products_quantity ) ) ),
			'upsell_ids'            => array_map( 'absint', $product->get_upsell_ids() ),
			'cross_sell_ids'        => array_map( 'absint', $product->get_cross_sell_ids() ),
			'parent_id'             => $product->is_type( 'variation' ) ? $product->parent->id : $post_object->post_parent,
			'categories'            => self::get_product_taxonomy_terms( $product ),
			'images'                => self::get_product_images( $product, $image_size ),
			'attributes'            => self::get_product_attributes( $product ),
		);
		
		if( $type == 'single' ) {
			$short_description = apply_filters( 'woocommerce_short_description', $post_object->post_excerpt );
			$towxml = new ToWXML();
			
			$data['description'] 			= $towxml->towxml( wpautop( do_shortcode( $post_object->post_content ) ), array( 'encode' => false ) );
			$data['short_description'] 		= $towxml->towxml( $short_description, array( 'encode' => false ) );
			$data['short_description_text'] = self::strip_html_tags( array( 'p', 'span', 'img', 'strong' ), $short_description );
			$data['downloads'] 				= self::get_product_downloads( $product );
			$data['purchase_note'] 			= wpautop( do_shortcode( wp_kses_post( $product->get_purchase_note() ) ) );
			$data['tags'] 					= self::get_product_taxonomy_terms( $product, 'tag' );
			$data['default_attributes'] 	= self::get_product_default_attributes( $product );
			$data['variations'] 			= array();
			$data['grouped_products'] 		= array();
			$data['menu_order'] 			= self::get_product_menu_order( $product );
		}

		return $data;
	}
	
	// 获取产品图片
	/**
	 * Get the images for a product or product variation.
	 *
	 * @param WC_Product|WC_Product_Variation $product
	 * @return array
	 */
	public static function get_product_images( $product, $size = 'full' ) {
		$images        = $attachment_ids = array();
		$product_image = $product->get_image_id();

		// Add featured image.
		if ( ! empty( $product_image ) ) {
			$attachment_ids[] = $product_image;
		}

		// Add gallery images.
		$attachment_ids = array_merge( $attachment_ids, $product->get_gallery_image_ids() );

		// Build image data.
		foreach ( $attachment_ids as $position => $attachment_id ) {
			$attachment_post = get_post( $attachment_id );
			if ( is_null( $attachment_post ) ) {
				continue;
			}

			$attachment = wp_get_attachment_image_src( $attachment_id, 'full' );
			if ( ! is_array( $attachment ) ) {
				continue;
			}

			/*$images[] = array(
				//'id'             => (int) $attachment_id,
				//'date_created'   => wc_rest_prepare_date_response( $attachment_post->post_date_gmt ),
				//'date_modified'  => wc_rest_prepare_date_response( $attachment_post->post_modified_gmt ),
				//'name'           => get_the_title( $attachment_id ),
				//'alt'            => get_post_meta( $attachment_id, '_wp_attachment_image_alt', true ),
				//'position'       => (int) $position,
			);*/
			
			switch( $size ) {
				case 'full':
					$images[] = array( 'src' => current( $attachment ) );
					break;
				case 'single':
					$images[] = array( 'shop_single' => current( wp_get_attachment_image_src( $attachment_id, 'shop_single') ) );
					break;
				case 'thumbnail':
					$images[] = array( 'shop_thumbnail' => current( wp_get_attachment_image_src( $attachment_id, 'shop_thumbnail') ) );
					break;
				default:
					$images[] = array( 'src' => current( $attachment ) );
			}
		}

		// Set a placeholder image if the product has no images set.
		if ( empty( $images ) ) {
			$images[] = array(
				//'id'            => 0,
				//'date_created'  => wc_rest_prepare_date_response( current_time( 'mysql' ) ), // Default to now.
				//'date_modified' => wc_rest_prepare_date_response( current_time( 'mysql' ) ),
				'src'           => wc_placeholder_img_src(),
				'shop_single'     => wc_placeholder_img_src(),
				'shop_thumbnail'     => wc_placeholder_img_src(),
				//'name'          => __( 'Placeholder', 'woocommerce' ),
				//'alt'           => __( 'Placeholder', 'woocommerce' ),
				//'position'      => 0,
			);
		}

		return $images;
	}
	
	// 获取产品分类
	/**
	 * Get taxonomy terms.
	 *
	 * @param WC_Product $product
	 * @param string $taxonomy
	 * @return array
	 */
	public static function get_product_taxonomy_terms( $product, $taxonomy = 'cat' ) {
		$terms = array();

		foreach ( wp_get_post_terms( $product->get_id(), 'product_' . $taxonomy ) as $term ) {
			$terms[] = array(
				'id'   => $term->term_id,
				'name' => $term->name,
				'slug' => $term->slug,
			);
		}

		return $terms;
	}
	
	// 获取属性分类名
	/**
	 * Get attribute taxonomy label.
	 *
	 * @param  string $name
	 * @return string
	 */
	public static function get_attribute_taxonomy_label( $name ) {
		$tax    = get_taxonomy( $name );
		$labels = get_taxonomy_labels( $tax );

		return $labels->singular_name;
	}
	
	// 获取产品默认属性
	/**
	 * Get default attributes.
	 *
	 * @param WC_Product $product
	 * @return array
	 */
	public static function get_product_default_attributes( $product ) {
		$default = array();

		if ( $product->is_type( 'variable' ) ) {
			foreach ( array_filter( (array) get_post_meta( $product->get_id(), '_default_attributes', true ), 'strlen' ) as $key => $value ) {
				if ( 0 === strpos( $key, 'pa_' ) ) {
					$default[ 'attribute_' . $key ] = array(
						'id'			=> wc_attribute_taxonomy_id_by_name( $key ),
						'name'			=> slef::get_attribute_taxonomy_label( $key ),
						'is_taxonomy'	=> true,
						'option'		=> $value,
					);
				} else {
					$default[ 'attribute_' . $key ] = array(
						'id'			=> 0,
						'name'			=> str_replace( 'pa_', '', $key ),
						'is_taxonomy'	=> true,
						'option'		=> $value,
					);
				}
			}
		}

		return $default;
	}
	
	// 获取产品属性选项
	/**
	 * Get attribute options.
	 *
	 * @param int $product_id
	 * @param array $attribute
	 * @return array
	 */
	public static function get_product_attribute_options( $product_id, $attribute ) {
		if ( isset( $attribute['is_taxonomy'] ) && $attribute['is_taxonomy'] ) {
			return wc_get_product_terms( $product_id, $attribute['name'], array( 'fields' => 'all' ) );
		} elseif ( isset( $attribute['value'] ) ) {
			$options = array();
			foreach( explode( '|', $attribute['value'] ) as $value ) {
				$options[] = array(
					'name' => trim( $value ),
					'slug' => trim( $value )
				);
			}
			return $options;
		}

		return array();
	}
	
	// 获取产品属性
	/**
	 * Get the attributes for a product or product variation.
	 *
	 * @param WC_Product|WC_Product_Variation $product
	 * @return array
	 */
	public static function get_product_attributes( $product ) {
		$attributes = array();

		if ( $product->is_type( 'variation' ) ) {
			// Variation attributes.
			foreach ( $product->get_variation_attributes() as $attribute_name => $attribute ) {
				$name = str_replace( 'attribute_', '', $attribute_name );

				// Taxonomy-based attributes are prefixed with `pa_`, otherwise simply `attribute_`.
				if ( 0 === strpos( $attribute_name, 'attribute_pa_' ) ) {
					$attributes[ $attribute_name ] = array(
						'id'     => wc_attribute_taxonomy_id_by_name( $name ),
						'name'   => self::get_attribute_taxonomy_label( $name ),
						'option' => $attribute,
					);
				} else {
					$attributes[ $attribute_name ] = array(
						'id'     => 0,
						'name'   => str_replace( 'pa_', '', $name ),
						'option' => $attribute,
					);
				}
			}
		} else {
			foreach ( $product->get_attributes() as $slug => $attribute ) {
				if ( $attribute['is_taxonomy'] ) {
					$attributes[ 'attribute_' . $slug ] = array(
						'id'        => wc_attribute_taxonomy_id_by_name( $attribute['name'] ),
						'name'      => self::get_attribute_taxonomy_label( $attribute['name'] ),
						'slug'		=> 'attribute_' . $slug,
						'is_taxonomy'	=> (bool) $attribute['is_taxonomy'],
						'position'  => (int) $attribute['position'],
						'visible'   => (bool) $attribute['is_visible'],
						'variation' => (bool) $attribute['is_variation'],
						'options'   => self::get_product_attribute_options( $product->get_id(), $attribute ),
					);
				} else {
					$attributes[ 'attribute_' . $slug ] = array(
						'id'        => 0,
						'name'      => str_replace( 'pa_', '', $attribute['name'] ),
						'slug'		=> 'attribute_' . $slug,
						'is_taxonomy'	=> (bool) $attribute['is_taxonomy'],
						'position'  => (int) $attribute['position'],
						'visible'   => (bool) $attribute['is_visible'],
						'variation' => (bool) $attribute['is_variation'],
						'options'   => self::get_product_attribute_options( $product->get_id(), $attribute ),
					);
				}
			}
		}

		return $attributes;
	}
	
	// 获取文章图片
	public static function get_post_image( $post_id, $size = 'full' ) {
		$attachment_id = get_post_thumbnail_id( $post_id );
		$attachment_post = get_post( $attachment_id );
		if ( is_null( $attachment_post ) ) {
			return false;
		}
		$attachment = wp_get_attachment_image_src( $attachment_id, 'full' );
		if ( ! is_array( $attachment ) ) {
			return false;
		}
		
		/*$image = array(
			//'id'            => (int) $attachment_id,
			//'date_created'  => wc_rest_prepare_date_response( $attachment_post->post_date_gmt ),
			//'date_modified' => wc_rest_prepare_date_response( $attachment_post->post_modified_gmt ),
			'src'           => current( $attachment ),
			'medium_large'  => current( wp_get_attachment_image_src( $attachment_id, 'medium_large') ),
			//'name'          => get_the_title( $attachment_id ),
			//'alt'           => get_post_meta( $attachment_id, '_wp_attachment_image_alt', true )
		);*/
		
		switch( $size ) {
			case 'full':
				$image = array( 'src' => current( $attachment ) );
				break;
			case 'large':
				$image = array( 'medium_large' => current( wp_get_attachment_image_src( $attachment_id, 'medium_large') ) );
				break;
			default:
				$image = array( 'src' => current( $attachment ) );
		}
		
		return $image;
	}
	
	// 获取小程序二维码
	public static function get_qrcode( $id, $type = 'product', $width = 400 ) {
		$types = array(
			'product' => 'pages/product-detail/product-detail',
			'post' => 'pages/post-detail/post-detail',
		);
		
		if( ! array_key_exists( $type, $types ) ) {
			return null;
		}
		
		$upload_dir = wp_upload_dir();
		$qrcode_dir = $upload_dir['basedir'] . '/w2w_qrcode/';
		if( ! is_dir( $qrcode_dir ) ) wp_mkdir_p( $qrcode_dir );
		
		$qrcode_filename = "$type-$id-$width.jpg";
		$qrcode_file = $qrcode_dir . $qrcode_filename;
		$path = "{$types[$type]}?id=$id";
		
		if( ! file_exists( $qrcode_file ) ) {

			$qrcode_image = W2W()->wxapi->get_qrcode( $path, $width );
			file_put_contents( $qrcode_file, $qrcode_image );
		}
		
		return array(
			'url' => $upload_dir['baseurl'] . '/w2w_qrcode/' . $qrcode_filename,
			'path' => $path,
			'width' => $width
		);
	}
	
	// 获取订单物品描述
	public static function get_goods_description( $order ) {
		
		$items = $order->get_items();
		$items_count = 0;
		$first_item = current( $items );
		$goods_desciption = $first_item['name'];
		
		foreach( $items as $item ) {
			$items_count += wc_stock_amount( $item['qty'] );
		}
		
		$goods_desciption .= ( $items_count > 1 ? ' 等' . $items_count . '件物品' : '' );
		return $goods_desciption;
	}
	
	// 获取小程序中支持的支付方式
	public static function get_supported_payment_methods() {
		$payments = array(
			'wxapay',
			'cod',
			'bacs',
			'bacs_china'
		);
		return apply_filters( 'w2w_available_payments', $payments );
	}
	
	// 获取产品列表排除字段
	public static function get_product_list_exclude_fields() {
		$fields = array(
			'description',
			'short_description',
			'variations',
			'related_ids',
			'manage_stock',
			'reviews_allowed',
			'tax_class',
			'tax_status',
			'virtual',
			'upsell_ids',
			'weight',
			'_links',
			'dimensions',
			'button_text',
			'cross_sell_ids',
			'download_expiry',
			'download_limit',
			'download_type',
			'downloadable',
			'downloads',
			'grouped_products',
			'sold_individually',
			'shipping_taxable',
			'shipping_class',
			'shipping_class_id',
			'shipping_required',
			'date_created',
			'date_modified',
			'date_on_sale_from',
			'date_on_sale_to',
		);
		
		return apply_filters( 'w2w_product_list_exclude_fields', $fields );
	}
	
	// 获取文章列表排除字段
	public static function get_post_list_exclude_fields() {
		$fields = array(
			'comment_status',
			'content',
			'format',
			'link',
			'meta',
			'status',
			'type',
		);
		
		return apply_filters( 'w2w_post_list_exclude_fields', $fields );
	}
}