<?php

if ( ! defined( 'ABSPATH' ) ) exit;

require_once( WP_W2W_PATH . 'includes/wxpay/class.WxPay.Api.php' );
require_once( WP_W2W_PATH . 'includes/wxpay/class.WxPay.JsApiPay.php' );
require_once( WP_W2W_PATH . 'includes/wxpay/class.WxPay.Notify.php' );

class W2W_WC_Gateway_WeChatPay extends WC_Payment_Gateway {
	
	public function __construct() {
		
		$this->id = 'wxapay';
		$this->has_fields = false;
		$this->order_button_text  = '微信支付';
		$this->title = '微信支付';
		$this->method_title = '微信支付';
		$this->method_description = '小程序微信支付';
		$this->supports = array(
			'products',
			'refunds',
		);
		
		$this->init_form_fields();
		$this->init_settings();
		
		add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
		
		// 支付参数
		add_filter( 'w2w_get_payment_params', array( $this, 'get_payment_params' ), 10, 2 );
		
		// 支付通知
		add_action( 'w2w_payment_notify', array( $this, 'payment_notify' ) );
		
		// 查看订单详情时查询订单支付状态
		add_action( 'w2w_rest_order_get_item', array( $this, 'rest_get_order_item_query_order' ) );
	}
	
	// 设置字段
	public function init_form_fields() {
		$this->form_fields = array(
			'enabled' => array(
				'title' => '启用禁用',
				'type' => 'checkbox',
				'label' => '启用微信支付',
				'default' => 'yes'
			),
			'title' => array(
				'title' => '标题',
				'type' => 'text',
				'default' => '微信支付'
			),
			'description' => array(
				'title' => '描述',
				'type' => 'text',
				'default' => '小程序微信支付'
			),
			'mchid' => array(
				'title' => '商户号（必填）',
				'type' => 'text',
				'description' => '获取地址：<a href="https://pay.weixin.qq.com/index.php/core/account/info" target="_blank">https://pay.weixin.qq.com/index.php/core/account/info</a>，微信支付商户号',
			),
			'key' => array(
				'title' => '商户支付密钥（必填）',
				'type' => 'text',
				'description' => '获取地址：<a href="https://pay.weixin.qq.com/index.php/core/cert/api_cert" target="_blank">https://pay.weixin.qq.com/index.php/core/cert/api_cert</a>，API密钥-设置密钥',
			),
			'body' => array(
				'title' => '支付简要描述（必填）',
				'type' => 'text',
				'description' => '商家名称-销售商品类目，例如：腾讯-游戏。线上电商，商家名称必须为实际销售商品的商家',
			),
		);
	}
	
	// 支付处理
	function process_payment( $order_id ) {
		
		global $woocommerce;
		$order = new WC_Order( $order_id );

		// 添加订单备注
		$order->add_order_note( '等待微信支付' );

		// 减少库存
		$order->reduce_order_stock();

		// 清空购物车
		$woocommerce->cart->empty_cart();

		return array(
			'result' => 'success',
			'order' => $order->get_data()
		);
	}
	
	// 退款处理
	public function process_refund( $order_id, $amount = null, $reason = '' ) {
		$order = wc_get_order( $order_id );
		$refund_number = get_post_meta( $order_id, 'w2w_order_refund_number', true );
		if( ! $refund_number ) {
			$refund_number = date_i18n( 'YmdHis' );
			update_post_meta( $order_id, 'w2w_order_refund_number', $refund_number );
		}

		$input = new W2W_WxPayRefund();
		$input->SetTransaction_id( $order->get_transaction_id() );
		$input->SetTotal_fee( intval( strval( floatval( $order->get_total() ) * 100 ) ) );
		$input->SetRefund_fee( intval( strval( floatval( $amount ) * 100 ) ) );
		$input->SetOp_user_id( $this->get_option( 'mchid' ) );
	    $input->SetOut_refund_no( $refund_number );
	    $input->SetRefund_desc( $reason );
		
		$result = W2W_WxPayApi::refund( $input );
		
		if( $result['return_code'] == 'SUCCESS' && $result['result_code'] == 'SUCCESS' ) {
			delete_post_meta( $order_id, 'w2w_order_refund_number' );
			return true;
		}
		else {
			W2W()->log( 'error', '退款错误：' . json_encode( $result ) );
			return new WP_Error( 'w2w_refund_error', $result['err_code_des'] );
		}
	}
	
	// 获取支付参数
	public function get_payment_params( $result, $wc_order ) {
		$settings = get_option( 'w2w-settings' );
		$wxpay_mchid = $this->get_option( 'mchid' );
		$wxpay_key = $this->get_option( 'key' );
		$wxpay_body = $this->get_option( 'body' );
		
		if( empty( $settings['appid'] ) || empty( $wxpay_mchid ) || empty( $wxpay_key ) || empty( $wxpay_body ) ) {
			return array( 'success' => false, 'msg' => '请填写AppID、商户号、商户支付密钥和支付简要描述' );
		}

		$tools = new W2W_JsApiPay();
		$openid = W2W()->util->get_user_openid( wp_get_current_user() );

		$input = new W2W_WxPayUnifiedOrder();
		$input->SetBody( $wxpay_body );
		$input->SetOut_trade_no( get_post_meta( $wc_order->get_id(), 'w2w_order_number', true ) );
		$input->SetTotal_fee( intval( strval( floatval( $wc_order->get_total() ) * 100 ) ) );
		$input->SetTime_start( date_i18n( 'YmdHis' ) );
		$input->SetTime_expire( date( 'YmdHis', strtotime( date_i18n( 'YmdHis' ) ) + 600 ) );
		$input->SetNotify_url( get_rest_url( null, 'w2w/v1/payment/notify' ) );
		$input->SetTrade_type( 'JSAPI' );
		$input->SetOpenid( $openid );
		
		$order = W2W_WxPayApi::unifiedOrder( $input );
		
		// 存储prepay_id
		if( isset( $order['prepay_id'] ) ) {
			update_post_meta( $wc_order->get_id(), 'w2w_prepay_id', $order['prepay_id'] );
		}
		
		$js_api_params = $tools->GetJsApiParameters( $order );
		$js_api_params['success'] = ! isset( $js_api_params['return_code'] );
		
		return $js_api_params;
	}
	
	// 查看订单详情时查询订单支付状态
	public function rest_get_order_item_query_order( $order ) {
		if( $order->get_status() == 'pending' && $order->get_payment_method() == $this->id ) {
			$wechat_order = $this->query_order( get_post_meta( $order->get_id(), 'w2w_order_number', true ), 'wc_order' );
			if( $wechat_order['success'] ) {
				$order->payment_complete( $wechat_order['order']['transaction_id'] );
				do_action( 'w2w_order_paid', $order->get_id() );
			}
		}
	}
	
	// 支付通知
	public function payment_notify() {
		$notify = new W2W_PayNotifyCallBack();
		$notify->Handle( false );
	}
	
	// 查询订单
	public function query_order( $order_id, $type = 'wechat_order' ) {
		
		$input = new W2W_WxPayOrderQuery();
		if( $type == 'wechat_order' ) {
			$input->SetTransaction_id( $order_id );
		}
		else if( $type == 'wc_order' ) {
			$input->SetOut_trade_no( $order_id );
		}
		
		$result = W2W_WxPayApi::orderQuery( $input );
		W2W()->log( 'info', '订单查询: ' . json_encode( $result ) );
		
		if( array_key_exists( 'return_code', $result )
			&& array_key_exists( 'result_code', $result )
			&& $result['return_code'] == 'SUCCESS'
			&& $result['result_code'] == 'SUCCESS'
			&& $result['trade_state'] == 'SUCCESS'
			&& W2W()->util->check_notify_sign( $result, $this->get_option( 'key' ) ) )
		{
			$success = true;
		}
		else{
			$success = false;
		}
		
		return array( 'success' => $success, 'order' => $result );
	}
}

class W2W_PayNotifyCallBack extends W2W_WxPayNotify {
	
	// 重写回调处理函数
	public function NotifyProcess( $data, &$msg ) {
		
		W2W()->log( 'info', '支付通知: ' . json_encode( $data ) );
		
		if( ! array_key_exists( 'transaction_id' , $data ) ) {
			$msg = '输入参数不正确';
			return false;
		}
		
		if( ! W2W()->util->check_notify_sign( $data, $this->get_option( 'key' ) ) ) {
			return false;
		}
		
		// 更改订单状态
		$orders = wc_get_orders( array( 'w2w_order_number' => $data['out_trade_no'] ) );
		
		if( count( $orders ) > 0 ) {
			
			$order = $orders[0];
			
			if( $order->get_status() == 'pending' ) {
				
				//查询订单，判断订单真实性
				$wechat_order = $this->query_order( $data['transaction_id'] );
				if( ! $wechat_order['success'] ) {
					$msg = '订单查询失败';
					return false;
				}
				
				$order->payment_complete( $data['transaction_id'] );
				do_action( 'w2w_order_paid', $order->get_id() );
			}
		}
		
		return true;
	}
}